clc
clear

% load map

load EK10C0822.mat;
map = EK10C0822;

% load EK10C1220.mat;
% map = EK10C1220;

% load EK40C2429.mat
% map = EK40C2429;

% load EK40C3425.mat;
% map = EK40C3425;

% load EK40C4521.mat;
% map = EK40C4521;

% extract data from table and convert units
Nc  = map.n .* 1e3;
Wc  = map.flow;
PI  = map.pratio;
eff = map.etac;
Treference = 273.15 + 15;
preference = 101.3e3;

% Required signals:
NcCorr      	= Nc;           % [rpm]     Column vector with the corrected compressor speed
WcCorr       	= Wc;           % [kg/s]    Column vector with the corrected compressor mass flow
PiC            	= PI;           % [-]       Column vector with the compressor pressure ratio
etaC         	= eff;          % [-]       Column vector with the compressor efficiency
TCref        	= Treference;   % [K]       Compressor map reference temperature, scalar
pCref       	= preference;   % [Pa]      Compressor map reference pressure, scalar
% Optional signals:
D_2           	= 47e-3;    	% [m]       Compressor outer impeller diameter, scalar
% T01        	= [];         	% [K]       Column vector with the compressor inlet temperature
% p01         	= [];           % [Pa]   	Column vector with the compressor inlet pressure


% Opens the toolbox:
map  = get_mapStruct(NcCorr,WcCorr,PiC,etaC,TCref,pCref,D_2);
LiU_CPgui(map)


% After parameter fitting:
% To use the compressor parameter set in the ECCV platform:
% 1. Output the parameter struct from the toolbox, it is called "paramT"
% 2. Remove the function handle fields of paramT.
% 3. Rename "paramT" to "par"
% 4. Save the struct "par" to a suitable name and place in the folder "models/F0011_ellipseCompressor"
% 5. The ellipse compressor simulink block should now find the parameter set after updating the models.
% 6. See the examples of parameter structs in "models/F0011_ellipseCompressor" for guidance. 










